import typing

from nsj_rest_lib.dto.dto_base import DTOBase
from nsj_rest_lib.entity.entity_base import EntityBase
from nsj_rest_lib.exception import DTOListFieldConfigException


class DTOListField:
    _ref_counter = 0

    def __init__(
        self,
        dto_type: DTOBase,
        entity_type: EntityBase = None,
        related_entity_field: str = None,
        not_null: bool = False,
        min: int = None,
        max: int = None,
        validator: typing.Callable = None,
        dto_post_response_type: DTOBase = None,
        relation_key_field: str = None,
        service_name: str = None,
    ):
        """
        -----------
        Parameters:
        -----------
        dto_type: Expected type for the related DTO (must be subclasse from DTOBase).
        entity_type: Expected entity type for the related DTO (must be subclasse from EntityBase).
        not_null: The field cannot be None (or an empty list).
        min: Minimum number of itens in the list.
        max: Maximum number of itens in the list.
        validator: Function that receives the value (to be setted), and returns the same value (after any adjust).
            This function overrides the default behaviour and all default constraints.
        related_entity_field: Fields, from related entity, used for relation in database.
        relation_key_field: Nome do campo, no DTO corrente, utilizado como chave de apontamento no relacionamento
            (isso é, campo para o qual a entidade, do lado N, aponta via FK).
        service_name: Nome do serviço customizado, caso se deseje que as operações sobre esse tipo de lista se façam
            de um modo customizado (e não usando o service_base do próprio RestLib).
        """
        self.name = None
        self.dto_type = dto_type
        self.entity_type = entity_type
        self.related_entity_field = related_entity_field
        self.not_null = not_null
        self.min = min
        self.max = max
        self.validator = validator
        self.dto_post_response_type = dto_post_response_type
        self.relation_key_field = relation_key_field
        self.service_name = service_name

        self.storage_name = f"_{self.__class__.__name__}#{self.__class__._ref_counter}"
        self.__class__._ref_counter += 1

        # Checking correct usage
        if self.dto_type is None:
            raise DTOListFieldConfigException("type parameter must be not None.")

        if service_name is None:
            if self.entity_type is None:
                raise DTOListFieldConfigException(
                    "entity_type parameter must be not None."
                )

    def __get__(self, instance, owner):
        if instance is None:
            return self
        else:
            return instance.__dict__[self.storage_name]

    def __set__(self, instance, value):
        try:
            if self.validator is None:
                value = self.validate(value)
            else:
                value = self.validator(value)
        except ValueError:
            if not (
                "escape_validator" in instance.__dict__
                and instance.__dict__["escape_validator"] is True
            ):
                raise

        # Preenchendo os campos de particionanmento, se necessário (normalmente: tenant e grupo_empresarial)
        self.set_partition_fields(instance, value)

        instance.__dict__[self.storage_name] = value

    def set_partition_fields(self, instance, value):
        """
        Preenchendo os campos de particionanmento dos objetos da lista, se necessário (normalmente: tenant e grupo_empresarial).
        """

        if hasattr(instance.__class__, "partition_fields") and value is not None:
            for item in value:
                for partition_field in instance.__class__.partition_fields:
                    if (
                        hasattr(instance, partition_field)
                        and hasattr(item, partition_field)
                        # TODO Analisar se devo descomentar a comparação abaixo que deixa gravar com campos de partição
                        # diferentes entre classe mestre e detalhe (caso se especifique diferente no detalhe)
                        # Talvez falte aqui a comparação de tipo de relacionamento como composição (quando não é composição
                        # a diferença pode fazer sentido)
                        # and getattr(item, partition_field) is None
                        and getattr(instance, partition_field)
                        != getattr(item, partition_field)
                    ):
                        partition_value = getattr(instance, partition_field)
                        setattr(item, partition_field, partition_value)

    def validate(self, value):
        """
        Default validator (ckecking default constraints: not null, type, min and max).
        """

        # Checking not null constraint
        if (self.not_null) and (
            value is None or (isinstance(value, list) and len(value) <= 0)
        ):
            raise ValueError(
                f"O campo {self.storage_name} deve ser preechido. Valor recebido: {value}."
            )

        # Checking if received value is a list
        if value is not None and not isinstance(value, list):
            raise ValueError(
                f"O valor recebido para o campo {self.storage_name} deveria ser uma lista. Valor recebido: {value}."
            )

        # Checking type constraint
        # TODO Ver como suportar typing
        if (
            self.dto_type is not None
            and value is not None
            and len(value) > 0
            and not isinstance(value[0], self.dto_type)
        ):
            raise ValueError(
                f"Os items da lista {self.storage_name} deveriam se do tipo {self.dto_type.__name__}. Valor recebido: {value}."
            )

        # Checking min constraint
        if self.min is not None and (value is None or len(value) < self.min):
            raise ValueError(
                f"A lista {self.storage_name} deve ter mais do que {self.min} itens. Valor recebido: {value}."
            )

        # Checking max constraint
        if self.max is not None and value is not None and len(value) > self.max:
            raise ValueError(
                f"A lista {self.storage_name} deve ter menos do que {self.max} itens. Valor recebido: {value}."
            )

        return value



import datetime
import enum
import re
import uuid

from decimal import Decimal
from typing import Any


class TypeValidatorUtil:
    @staticmethod
    def validate(obj, value):
        """
        Valida o value recebido, de acordo com o tipo esperado, e faz as conversões necessárias (se possível).
        """

        # Montando expressões regulares para as validações
        matcher_uuid = re.compile(
            "^[A-Fa-f0-9]{8}-[A-Fa-f0-9]{4}-[A-Fa-f0-9]{4}-[A-Fa-f0-9]{4}-[A-Fa-f0-9]{12}$"
        )
        matcher_datetime = re.compile(
            "^(\d\d\d\d)-(\d\d)-(\d\d)[T,t](\d\d):(\d\d):(\d\d)$"
        )
        matcher_date = re.compile("^(\d\d\d\d)-(\d\d)-(\d\d)$")

        matcher_time = re.compile("^(\d\d):(\d\d):(\d\d)$")

        # Validação direta de tipos
        erro_tipo = False
        if obj.expected_type is datetime.datetime and isinstance(value, str):
            match_datetime = matcher_datetime.search(value)
            match_date = matcher_date.search(value)

            if match_datetime:
                ano = int(match_datetime.group(1))
                mes = int(match_datetime.group(2))
                dia = int(match_datetime.group(3))
                hora = int(match_datetime.group(4))
                minuto = int(match_datetime.group(5))
                segundo = int(match_datetime.group(6))

                value = datetime.datetime(
                    year=ano,
                    month=mes,
                    day=dia,
                    hour=hora,
                    minute=minuto,
                    second=segundo,
                )
            elif match_date:
                ano = int(match_date.group(1))
                mes = int(match_date.group(2))
                dia = int(match_date.group(3))

                value = datetime.datetime(
                    year=ano, month=mes, day=dia, hour=0, minute=0, second=0
                )
            else:
                erro_tipo = True
        elif obj.expected_type is datetime.date and isinstance(value, str):
            match_date = matcher_date.search(value)

            if match_date:
                ano = int(match_date.group(1))
                mes = int(match_date.group(2))
                dia = int(match_date.group(3))

                value = datetime.date(year=ano, month=mes, day=dia)
            else:
                erro_tipo = True
        elif obj.expected_type is datetime.time and isinstance(value, str):
            match_time = matcher_time.search(value)
            if match_time:
                hor = int(match_time.group(1))
                min = int(match_time.group(2))
                sec = int(match_time.group(3))

                value = datetime.time(hour=hor, minute=min, second=sec)
            else:
                erro_tipo = True
        elif isinstance(obj.expected_type, enum.EnumMeta):
            # Enumerados
            try:
                value = TypeValidatorUtil.convert_enum_from_entity(obj, value)
            except ValueError:
                raise ValueError(
                    f"{obj.storage_name} não é um {obj.expected_type.__name__} válido. Valor recebido: {value}."
                )
        elif obj.expected_type is bool and isinstance(value, int):
            # Booleanos
            # Converting int to bool (0 is False, otherwise is True)
            value = bool(value)
        elif obj.expected_type is bool and isinstance(value, str):
            # Booleanos
            # Converting str to bool
            value = value.lower() == "true"
        elif obj.expected_type is datetime.datetime and isinstance(
            value, datetime.date
        ):
            # Datetime
            # Assumindo hora 0, minuto 0 e segundo 0 (quanto é recebida uma data para campo data + hora)
            value = datetime.datetime(value.year, value.month, value.day, 0, 0, 0)
        elif obj.expected_type is datetime.date and isinstance(
            value, datetime.datetime
        ):
            # Date
            # Desprezando hora , minuto e segundo (quanto é recebida uma data + hora, para campo de data)
            value = datetime.date(value.year, value.month, value.day)
        elif obj.expected_type is uuid.UUID and isinstance(value, str):
            # UUID
            # Verificando se pode ser alterado de str para UUID
            match_uuid = matcher_uuid.search(value)

            if match_uuid:
                value = uuid.UUID(value)
            else:
                erro_tipo = True
        elif obj.expected_type is int:
            # Int
            try:
                value = int(value)
            except:
                erro_tipo = True
        elif obj.expected_type is float:
            # Float
            try:
                value = float(value)
            except:
                erro_tipo = True
        elif obj.expected_type is Decimal:
            # Decimal
            try:
                value = Decimal(str(value))
            except:
                erro_tipo = True
        elif obj.expected_type is str:
            # String
            if value is not None:
                try:
                    value = str(value)
                except:
                    erro_tipo = True
        else:
            erro_tipo = True

        if erro_tipo:
            raise ValueError(
                f"{obj.storage_name} deve ser do tipo {obj.expected_type.__name__}. Valor recebido: {value}."
            )

        return value

    @staticmethod
    def convert_enum_from_entity(obj, value: Any):
        lista_enum = list(obj.expected_type)

        # Se o enum estiver vazio
        if len(lista_enum) <= 0:
            return None

        # Verificando o tipo dos valores do enum
        if isinstance(lista_enum[0].value, tuple):
            for item in obj.expected_type:
                lista_valores = list(item.value)
                for valor in lista_valores:
                    # Testando se casa com o valor
                    if valor == value:
                        return item

                    # Se o valor for string, testa inclusive em caixa alta e baixa
                    if isinstance(value, str):
                        if valor == value.lower() or valor == value.upper():
                            return item
            raise ValueError
        else:
            # Tentando pelo valor do próprio enum (e testando os casos, se for str)
            if isinstance(value, str):
                try:
                    return obj.expected_type(value)
                except ValueError:
                    try:
                        return obj.expected_type(value.lower())
                    except ValueError:
                        return obj.expected_type(value.upper())
            else:
                return obj.expected_type(value)



import enum


class FilterOperator(enum.Enum):
    EQUALS = "equals"
    DIFFERENT = "diferent"
    GREATER_THAN = "greater_than"
    LESS_THAN = "less_than"
    GREATER_OR_EQUAL_THAN = "greater_or_equal_than"
    LESS_OR_EQUAL_THAN = "less_or_equal_than"
    LIKE = "like"
    ILIKE = "ilike"
    NOT_NULL = "not_null"



import typing

from nsj_rest_lib.dto.dto_base import DTOBase
from nsj_rest_lib.entity.entity_base import EntityBase
from nsj_rest_lib.exception import DTOListFieldConfigException

from nsj_rest_lib.util.type_validator_util import TypeValidatorUtil


class EntityRelationOwner:
    SELF = "self"
    OTHER = "other"


class DTOLeftJoinField:
    _ref_counter = 0

    def __init__(
        self,
        resume: bool,
        dto_type: DTOBase,
        entity_type: EntityBase,
        related_dto_field: str,
        relation_field: str,
        entity_relation_owner: EntityRelationOwner = EntityRelationOwner.SELF,
        type: object = None,
        validator: typing.Callable = None,
        use_default_validator: bool = True,
    ):
        """
        -----------
        Parameters:
        -----------
        resume: O campo será usado como resumo, isto é, será sempre rotornado num HTTP GET que liste
            os dados (mesmo que não seja solicitado por meio da query string "fields").
        dto_type: Expected type for the related DTO (must be subclasse from DTOBase).
        entity_type: Expected entity type for the related DTO (must be subclasse from EntityBase).
        related_dto_field: Nome do campo, no DTO relacionado, a ser copiado para esse campo.
        relation_field: Nome do campo, usado na query, para correlacionar as entidades (correspondete
            ao campo usado no "on" de um "join").
        entity_relation_owner: Indica qual entidade contém o campo que aponta o relacionamento (
            se for EntityRelationField.OTHER, implica que a entidade apontada pela classe de DTO
            passada no decorator, é que contem o campo; se for o EntityRelationField.SELF, indica
            que o próprio DTO que contém o campo).
        type: Tipo esperado para a propriedade. Se for do tipo enum.Enum, o valor recebido, para
            atribuição à propriedade, será convertido para o enumerado.
        validator: Função que recebe o valor (a ser atribuído), e retorna o mesmo valor após algum
            tipo de tratamento (como adição ou remoção, automática, de formatação).
        use_default_validator: Flag indicando se o validator padrão deve ser aplicado à propriedade
            (esse validator padrão verifica o tipo de dados passado, e as demais verificações
            recebidas no filed, como, por exemplo, valor máximo, mínio, not_null, etc).
        """
        self.name = None
        self.resume = resume
        self.expected_type = type
        self.dto_type = dto_type
        self.entity_type = entity_type
        self.related_dto_field = related_dto_field
        self.relation_field = relation_field
        self.entity_relation_owner = entity_relation_owner
        self.validator = validator
        self.use_default_validator = use_default_validator

        self.storage_name = f"_{self.__class__.__name__}#{self.__class__._ref_counter}"
        self.__class__._ref_counter += 1

        # Checking correct usage
        if self.dto_type is None:
            raise DTOListFieldConfigException("type parameter must be not None.")

        if self.entity_type is None:
            raise DTOListFieldConfigException("entity_type parameter must be not None.")

    def __get__(self, instance, owner):
        if instance is None:
            return self
        else:
            return instance.__dict__[self.storage_name]

    def __set__(self, instance, value):
        try:
            if self.validator is None:
                if self.use_default_validator and value is not None:
                    value = TypeValidatorUtil.validate(self, value)
            else:
                if self.use_default_validator and value is not None:
                    value = TypeValidatorUtil.validate(self, value)
                value = self.validator(self, value)
        except ValueError as e:
            if not (
                "escape_validator" in instance.__dict__
                and instance.__dict__["escape_validator"] == True
            ):
                raise

        instance.__dict__[self.storage_name] = value


class LeftJoinQuery:
    related_dto: DTOBase
    related_entity: EntityBase
    fields: list[str]
    left_join_fields: list[DTOLeftJoinField]
    entity_relation_owner: EntityRelationOwner

    def __init__(self) -> None:
        self.related_dto: DTOBase = None
        self.related_entity: EntityBase = None
        self.fields: list[str] = []
        self.left_join_fields: list[DTOLeftJoinField] = []
        self.entity_relation_owner: EntityRelationOwner = None



import os

from flask import request
from typing import Callable

from nsj_rest_lib.controller.controller_util import DEFAULT_RESP_HEADERS
from nsj_rest_lib.controller.route_base import RouteBase
from nsj_rest_lib.dto.dto_base import DTOBase
from nsj_rest_lib.entity.entity_base import EntityBase
from nsj_rest_lib.exception import (
    DataOverrideParameterException,
    MissingParameterException,
)
from nsj_rest_lib.injector_factory_base import NsjInjectorFactoryBase
from nsj_rest_lib.settings import get_logger, DEFAULT_PAGE_SIZE

from nsj_gcf_utils.json_util import json_dumps
from nsj_gcf_utils.pagination_util import page_body, PaginationException
from nsj_gcf_utils.rest_error_util import format_json_error


class ListRoute(RouteBase):
    def __init__(
        self,
        url: str,
        http_method: str,
        dto_class: DTOBase,
        entity_class: EntityBase,
        injector_factory: NsjInjectorFactoryBase = NsjInjectorFactoryBase,
        service_name: str = None,
        handle_exception: Callable = None,
    ):
        super().__init__(
            url=url,
            http_method=http_method,
            dto_class=dto_class,
            entity_class=entity_class,
            dto_response_class=None,
            injector_factory=injector_factory,
            service_name=service_name,
            handle_exception=handle_exception,
        )

    def handle_request(
        self,
        query_args: dict[str, any] = None,
        body: dict[str, any] = None,
    ):
        """
        Tratando requisições HTTP Get (para listar entidades, e não para recuperar pelo ID).
        """

        with self._injector_factory() as factory:
            try:
                # Recuperando os parâmetros básicos
                if os.getenv("ENV", "").lower() != "erp_sql":
                    base_url = request.base_url
                    args = request.args
                else:
                    base_url = ""
                    args = query_args

                limit = int(args.get("limit", DEFAULT_PAGE_SIZE))
                current_after = args.get("after") or args.get("offset")

                # Tratando dos fields
                fields = args.get("fields")
                fields = RouteBase.parse_fields(self._dto_class, fields)
                url_args = (
                    base_url
                    + "?"
                    + "&".join(
                        [
                            f"{arg}={value}"
                            for arg, value in args.items()
                            if arg not in ["limit", "after", "offset"]
                        ]
                    )
                )

                # Tratando dos filters e search_query
                filters = {}
                search_query = None
                for arg in args:
                    if arg.lower() == "search":
                        search_query = args.get(arg)
                        continue

                    if arg in ["limit", "after", "offset", "fields"]:
                        continue

                    if arg in self._dto_class.partition_fields:
                        continue

                    filters[arg] = args.get(arg)

                # Tratando campos de particionamento
                for field in self._dto_class.partition_fields:
                    value = args.get(field)
                    if value is None:
                        raise MissingParameterException(field)

                    filters[field] = value

                # Tratando dos campos de data_override
                self._validade_data_override_parameters(args)

                # Construindo os objetos
                service = self._get_service(factory)

                # Chamando o service (método list)
                # TODO Rever parametro order_fields abaixo
                data = service.list(
                    current_after,
                    limit,
                    fields,
                    None,
                    filters,
                    search_query=search_query,
                )

                # Convertendo para o formato de dicionário (permitindo omitir campos do DTO)
                dict_data = [dto.convert_to_dict(fields) for dto in data]

                # Recuperando o campo referente à chave primária do DTO
                pk_field = self._dto_class.pk_field

                # Construindo o corpo da página
                page = page_body(
                    base_url=url_args,
                    limit=limit,
                    current_after=current_after,
                    current_before=None,
                    result=dict_data,
                    id_field=pk_field,
                )

                # Retornando a resposta da requuisição
                return (json_dumps(page), 200, {**DEFAULT_RESP_HEADERS})
            except MissingParameterException as e:
                get_logger().warning(e)
                if self._handle_exception is not None:
                    return self._handle_exception(e)
                else:
                    return (format_json_error(e), 400, {**DEFAULT_RESP_HEADERS})
            except DataOverrideParameterException as e:
                get_logger().warning(e)
                if self._handle_exception is not None:
                    return self._handle_exception(e)
                else:
                    return (format_json_error(e), 400, {**DEFAULT_RESP_HEADERS})
            except PaginationException as e:
                get_logger().warning(e)
                if self._handle_exception is not None:
                    return self._handle_exception(e)
                else:
                    return (format_json_error(e), 400, {**DEFAULT_RESP_HEADERS})
            except Exception as e:
                get_logger().exception(e)
                if self._handle_exception is not None:
                    return self._handle_exception(e)
                else:
                    return (
                        format_json_error(f"Erro desconhecido: {e}"),
                        500,
                        {**DEFAULT_RESP_HEADERS},
                    )



import functools

from typing import Any, List



class EntityField:
    expected_type: object


class Entity:
    def __init__(
        self,
        table_name: str,
        pk_field: str,
        default_order_fields: List[str]
    ) -> None:
        super().__init__()

        self.table_name = table_name
        self.pk_field = pk_field
        self.default_order_fields = default_order_fields

    def __call__(self, cls):
        """
        Tratando dos tipos de dados dos atributos, e criando os getters necessários.
        """

        # Mantém metadados da classe original
        functools.update_wrapper(self, cls)

        # Guardando o nome da tabela na classe
        self._check_class_attribute(cls, "table_name", self.table_name)

        # Guardando o nome do campo PK na classe
        self._check_class_attribute(cls, "pk_field", self.pk_field)

        # Guardando a lista default de ordenação, na classe
        self._check_class_attribute(
            cls, "default_order_fields", self.default_order_fields
        )

        # Creating fields_map in cls, if needed
        self._check_class_attribute(cls, "fields_map", {})

        # Iterando pelos atributos de classe
        for key, attr in cls.__dict__.items():
            # Copiando o tipo a partir da anotação de tipo (se existir)
            if key in cls.__annotations__:
                atributo = attr
                if not isinstance(attr, EntityField):
                    atributo = EntityField()

                atributo.expected_type = cls.__annotations__[key]

                # Guardando o atributo no fields_map
                getattr(cls, "fields_map")[key] = atributo

        return cls

    def _check_class_attribute(self, cls: object, attr_name: str, default_value: Any):
        """
        Add attribute "attr_name" in class "cls", if not exists.
        """

        if attr_name not in cls.__dict__:
            setattr(cls, attr_name, default_value)


import functools
from flask import request
from nsj_rest_lib.dto.dto_base import DTOBase
from nsj_rest_lib.descriptor.dto_field import DTOField
from nsj_rest_lib.decorator.opentelemetry_base import OpenTelemetryBase
from nsj_rest_lib.util.time_grouping import TimeGrouping

class OpenTelemetry:
    def __init__(
        self,
        dto_class: DTOBase,
        route: str,
        metric_name: str,
        counter_name: str,
        description_counter: str = "",
        tenant_field: str = "tenant",
        grupo_empresarial_field: str = "grupo_empresarial",
        time_grouping: TimeGrouping = TimeGrouping.WEEK_OF_YEAR,
    ):
        self.decorator = OpenTelemetryBase(
            route=route,
            metric_name=metric_name,
            counter_name=counter_name,
            description_counter=description_counter,
            tenant_field=tenant_field,
            grupo_empresarial_field=grupo_empresarial_field,
            time_grouping=time_grouping,
        )
        self.dto_class = dto_class
        self.tenant_field = tenant_field
        self.grupo_empresarial_field = grupo_empresarial_field

    def __call__(self, func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            metric_fields = DTOField.get_metric_labels(
                self.dto_class,
                request,
                self.tenant_field,
                self.grupo_empresarial_field,
            ) if self.dto_class else {}

            self.decorator.extra_fields = metric_fields

            return self.decorator(func)(*args, **kwargs)

        return wrapper



import os
import base64
import requests

from healthcheck import HealthCheck

from nsj_rest_lib.injector_factory_base import NsjInjectorFactoryBase


class HealthCheckConfig:
    def __init__(
        self,
        flask_application,
        injector_factory_class: NsjInjectorFactoryBase = None,
        app_name: str = None,
        rabbitmq_host: str = None,
        rabbitmq_http_port: int = None,
        rabbitmq_user: str = None,
        rabbitmq_pass: str = None,
    ):
        self._flask_application = flask_application
        self._injector_factory_class = injector_factory_class or NsjInjectorFactoryBase
        self._app_name = app_name or os.getenv('APP_NAME')
        self._rabbitmq_host = rabbitmq_host or os.getenv('RABBITMQ_HOST')
        self._rabbitmq_http_port = rabbitmq_http_port or int(
            os.getenv('RABBITMQ_HTTP_PORT', 15672))
        self._rabbitmq_user = rabbitmq_user or os.getenv(
            'RABBITMQ_USER', 'guest')
        self._rabbitmq_pass = rabbitmq_pass or os.getenv(
            'RABBITMQ_PASS', 'guest')

    def check_database(self):
        with self._injector_factory_class() as factory:
            sql = "SELECT 1"

            factory.db_adapter().execute_query(sql)

            return True, "Banco de dados OK"

    def check_rabbit_mq(self):
        rabbit_url = f"{self._rabbitmq_host}:{self._rabbitmq_http_port}/api/healthchecks/node"
        if rabbit_url[0:4] != 'http':
            rabbit_url = 'http://' + rabbit_url

        credentials = f"{self._rabbitmq_user}:{self._rabbitmq_pass}"
        credentials = credentials.encode('utf8')
        credentials = base64.b64encode(credentials)
        credentials = credentials.decode('utf8')

        headers = {
            "Authorization": f"Basic {credentials}"
        }

        response = requests.get(rabbit_url, headers=headers)

        if response.status_code == 200:
            return True, "RabbitMQ OK"
        else:
            return False, f"Falha de comunicação com o RabbitMQ"

    def config(
        self,
        check_database: bool = True,
        check_rabbit_mq: bool = False
    ):

        health = HealthCheck()

        # Adicionando a validação de banco de dados
        if check_database:
            health.add_check(self.check_database)

        # Adicionando a validação do RabbitMQ
        if check_rabbit_mq:
            health.add_check(self.check_rabbit_mq)

        # Registrando a rota do HealthCheck
        if self._app_name is not None:
            self._flask_application.add_url_rule(f"/{self._app_name}/healthcheck", "healthcheck",
                                                 view_func=lambda: health.run())
        else:
            self._flask_application.add_url_rule(f"/healthcheck", "healthcheck",
                                                 view_func=lambda: health.run())



import os

from flask import request
from typing import Callable

from nsj_rest_lib.controller.controller_util import DEFAULT_RESP_HEADERS
from nsj_rest_lib.controller.route_base import RouteBase
from nsj_rest_lib.dto.dto_base import DTOBase
from nsj_rest_lib.entity.entity_base import EntityBase
from nsj_rest_lib.exception import (
    MissingParameterException,
    ConflictException,
)
from nsj_rest_lib.injector_factory_base import NsjInjectorFactoryBase
from nsj_rest_lib.settings import get_logger

from nsj_gcf_utils.json_util import json_dumps, JsonLoadException
from nsj_gcf_utils.rest_error_util import format_json_error


class PostRoute(RouteBase):
    def __init__(
        self,
        url: str,
        http_method: str,
        dto_class: DTOBase,
        entity_class: EntityBase,
        dto_response_class: DTOBase = None,
        injector_factory: NsjInjectorFactoryBase = NsjInjectorFactoryBase,
        service_name: str = None,
        handle_exception: Callable = None,
        custom_before_insert: Callable = None,
        custom_after_insert: Callable = None,
        retrieve_after_insert: bool = False,
    ):
        super().__init__(
            url=url,
            http_method=http_method,
            dto_class=dto_class,
            entity_class=entity_class,
            dto_response_class=dto_response_class,
            injector_factory=injector_factory,
            service_name=service_name,
            handle_exception=handle_exception,
        )
        self.custom_before_insert = custom_before_insert
        self.custom_after_insert = custom_after_insert
        self.retrieve_after_insert = retrieve_after_insert

    def _partition_filters(self, data):
        # Montando os filtros de particao de dados
        partition_filters = {}

        for field in data.partition_fields:
            value = getattr(data, field)
            if value is None:
                raise MissingParameterException(field)
            elif value is not None:
                partition_filters[field] = value

        return partition_filters

    def handle_request(
        self,
        query_args: dict[str, any] = None,
        body: dict[str, any] = None,
    ):
        """
        Tratando requisições HTTP Post para inserir uma instância de uma entidade.
        """

        with self._injector_factory() as factory:
            try:
                # Recuperando os dados do corpo da requisição
                if os.getenv("ENV", "").lower() != "erp_sql":
                    request_data = request.json
                else:
                    request_data = body

                if not isinstance(request_data, list):
                    request_data = [request_data]


                data_pack = []
                lst_data = []
                partition_filters = None
                for item in request_data:

                    # Convertendo os dados para o DTO
                    data = self._dto_class(validate_read_only=True, **item)

                    # Montando os filtros de particao de dados
                    if partition_filters is None:
                        partition_filters = self._partition_filters(data)

                    data_pack.append(data)


                # Construindo os objetos
                service = self._get_service(factory)

                if len(data_pack)==1:
                    # Chamando o service (método insert)
                    data = service.insert(
                        dto=data,
                        aditional_filters=partition_filters,
                        custom_before_insert=self.custom_before_insert,
                        custom_after_insert=self.custom_after_insert,
                        retrieve_after_insert=self.retrieve_after_insert,
                    )

                    if data is not None:
                        # Convertendo para o formato de dicionário (permitindo omitir campos do DTO)
                        lst_data.append(data.convert_to_dict())
                else:
                    data = service.insert_list(
                        dtos=data_pack,
                        aditional_filters=partition_filters,
                        custom_before_insert=self.custom_before_insert,
                        custom_after_insert=self.custom_after_insert,
                        retrieve_after_insert=self.retrieve_after_insert,
                    )

                    if data is not None or not len(data)>0:
                        # Convertendo para o formato de dicionário (permitindo omitir campos do DTO)
                        lst_data = [item.convert_to_dict() for item in data]

                if len(lst_data)==1:
                    # Retornando a resposta da requisição
                    return (json_dumps(lst_data[0]), 200, {**DEFAULT_RESP_HEADERS})

                if len(lst_data)>1:
                    # Retornando a resposta da requisição
                    return (json_dumps(lst_data), 200, {**DEFAULT_RESP_HEADERS})

                # Retornando a resposta da requisição
                return ("", 201, {**DEFAULT_RESP_HEADERS})
            except JsonLoadException as e:
                get_logger().warning(e)
                if self._handle_exception is not None:
                    return self._handle_exception(e)
                else:
                    return (format_json_error(e), 400, {**DEFAULT_RESP_HEADERS})
            except MissingParameterException as e:
                get_logger().warning(e)
                if self._handle_exception is not None:
                    return self._handle_exception(e)
                else:
                    return (format_json_error(e), 400, {**DEFAULT_RESP_HEADERS})
            except ValueError as e:
                get_logger().warning(e)
                if self._handle_exception is not None:
                    return self._handle_exception(e)
                else:
                    return (format_json_error(e), 400, {**DEFAULT_RESP_HEADERS})
            except ConflictException as e:
                get_logger().warning(e)
                if self._handle_exception is not None:
                    return self._handle_exception(e)
                else:
                    return (format_json_error(e), 409, {**DEFAULT_RESP_HEADERS})
            except Exception as e:
                get_logger().exception(e)
                if self._handle_exception is not None:
                    return self._handle_exception(e)
                else:
                    return (
                        format_json_error(f"Erro desconhecido: {e}"),
                        500,
                        {**DEFAULT_RESP_HEADERS},
                    )
